#!/bin/bash
#  download pdb file from PDB and finds gap hetero, metals and chains in the pdb file 
#  and printout relevant info 
# 
# syntax: 
#   ./find_gap.bash [-f] pdbcode
#    "pdbcode" is the alphanumeric code of the PDB file
#    if -f option is used; no wget from PDB ; file with pdbcode=xxxx 
#    [ i.e. pdbxxxx.ent] is assumed to be local  
#

if [ ! -z $ORAC_HOME  ] ;  then
    if [ -d $ORAC_HOME ] ; then
	DIRS=$ORAC_HOME
    else
	echo " ** ERROR ***"
	echo " given  ORAC_HOME variable "  $ORAC_HOME " does not look like the ORAC home"
	echo " set variable ORAC_HOME and retry "  
	exit 
    fi
else    
    echo "ORAC_HOME not set" 
    if [ -d ~/ORAC/trunk/src  ]; then 
	DIRS=~/ORAC/trunk  # default ORAC mount point  ; 
    elif [ -d ~/ORAC/src   ]; then  
	DIRS=~/ORAC  # default ORAC mount point  ; 
    else
	echo " ORAC_HOME not set; ORAC dir not found" 
	echo " set ORAC_HOME and try again"
    fi
fi 

local=0
while getopts ":fl" opt; do 
    case $opt in 
	f) 
	    local=1; 
	    ;;
	l) 
	    local=1; 
	    ;;
	\?) 
	    echo "invalid option"
	    ;;
    esac
done
code=${!#}

# wget the file if -f is not given 
if [ $local != 1 -a ! -s pdb${code}.ent ] ; then 
    wget ftp://ftp.wwpdb.org/pub/pdb/data/structures/all/pdb/pdb${code}.ent.gz;  
    if [ -s pdb${code}.ent.gz ]; then 
	gzip -d pdb${code}.ent.gz ; 
    else
	echo " ** ERROR ***"
	echo " wget failed! pdb file not found on ftp://ftp.wwpdb.org/pub/pdb/data/structures/all/pdb" 
	exit 
    fi
fi

# scanning for general info 
echo ""
echo " == GENERAL SECTION  "
printf "    NRESIDUE "; grep "ATOM " pdb${code}.ent | grep " CA " | wc -l  
printf "    DISSOCIATION CONSTANT = "; awk -v pdb=$code '{if($1==pdb) {printf "%8.2f\n", 0.596*$4*2.30259; found=1}}END{if(found!=1) print "n/a"}' $DIRS/lib/INDEX_general_PL_data.2016 ; 
awk '/TITLE/{print "   ", $0}' pdb${code}.ent
awk '/DOI/{print "   ", $0}' pdb${code}.ent
 
# scanning for gaps 
echo ""
echo " == GAP SECTION  "

grep "ATOM " pdb${code}.ent | grep " CA " | awk \
'{\
    ires=$6; dr=ires-ires_old;\
    if(dr>1) {\
       printf "%5d%5s%5d%5s  GAP%4d\n",  ires_old,name_old,ires,$4,dr\
    };\
    ires_old=$6;\
    name_old=$4\
}' 

# search for hetero and metals 
echo ""
echo " == HETERO ATOMS SECTION "

awk ' BEGIN{nres_old=-100} \
{\
    if($1=="HETATM"){ \
	i++;\
	het=substr($0,18,3); \
	nres=substr($0,23,5); \
	if(nres!=nres_old) {\
	    if(nres_old!=-100) { \
		j++;\
		hetero[j]=het_old
		netero[j]=i; \
	    }\
	    nres_old=nres;\
	    het_old=het;\
	    i=0\
	}\
    }\
}\
END {\
    nw=0;
    for(k=1; k<=j; k++) {\
	if(hetero[k]!="HOH") {\
		printf "    %3s natom=%3d\n",  hetero[k],netero[k]\
	} else {\
         nw++
        }\
    }\
    if(nw>0) { printf "    %6s n=%3d\n",  "WATER",nw}\
}' pdb${code}.ent 

for i in MN MG FE ZN CO CU CD NI W  MO VA; do 
    grep HETATM pdb${code}.ent  | grep -v HOH | grep -v REMARK | awk -v metal=$i\
    '{\
        atom=substr($0,77,2);\
        if(atom==metal) {\
            residue=substr($0,23,5); \
            print "--> METAAAAL!! ",atom,residue\
        }\
     }'
done

# scan for chains info
echo ""
echo " == CHAIN SECTION "
awk '{\
    if($1=="ATOM") {\
	{\
	    c=substr($0,22,1); \
	    n[c]+=1;\
	} \
    }\
}\
END {\
    if(n["A"]!=0) print "    chain A", n["A"];  \
    if(n["B"]!=0) print "    chain B", n["B"];  \
    if(n["C"]!=0) print "    chain C", n["C"];  \
    if(n["D"]!=0) print "    chain D", n["D"];  \
    if(n["E"]!=0) print "    chain E", n["A"];  \
    if(n["F"]!=0) print "    chain F", n["B"];  \
    if(n["G"]!=0) print "    chain G", n["C"];  \
    if(n["H"]!=0) print "    chain H", n["D"];  \
}' pdb${code}.ent

